package com.tanpu.fund.entity.generator;

import java.math.BigDecimal;
import java.util.Date;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * @author: zhoupeng
 * <p>
 * =========================================
 * =========================================
 * ========                         ========
 * =======   ==========   =======    =======
 * ======        =====    ==   ====   ======
 * =====        =====     ==    ===    =====
 * =====       =====      ==   ===     =====
 * =====      =====       == ===       =====
 * ======    ==========   ==          ======
 * =======                           =======
 * =========================================
 * =========================================
 * <p>
 * @email: zhoupeng_08@163.com
 */

/**
 * 基金历史业绩
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class FundCount {
    private String id;

    /**
     * 基金id
     */
    private String fundId;

    /**
     * 基金策略分类
     */
    private Integer strategy;

    /**
     * 融智子策略分类:1010-主观多头 1020-股票多空 1030-量化多头 2010-宏观策略 3010-主观趋势 3020-主观套利 3030-量化趋势 3040-量化套利 3050-管理期货复合 4010-并购重组 4020-定向增发 4030-大宗交易 4040-事件驱动复合 5010-股票市场中性 5020-套利 5030-相对价值复合 6010-纯债策略 6020-强债策略 6030-固收复合 7010-MOM 7020-FOF 8010-主观多策略 8020-量化多策略 -1-其他策略'
     */
    private Integer substrategy;

    /**
     * 基金成立日期
     */
    private Date inceptionDate;

    /**
     * 是否可预约 0:不可预约 1:可预约
     */
    private Integer type;

    /**
     * 截至日期
     */
    private Date endDate;

    /**
     * 最近累计净值日期
     */
    private Date priceDate;

    /**
     * 单位净值
     */
    private BigDecimal netNav;

    /**
     * 最近累计净值
     */
    private BigDecimal cumulativeNav;

    /**
     * 最近一天收益率
     */
    private BigDecimal ret1day;

    /**
     * 最近一天基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret1dayBm1;

    /**
     * 最近一个周收益率
     */
    private BigDecimal ret1w;

    /**
     * 最近一个周基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret1wBm1;

    /**
     * 最近一个月收益率
     */
    private BigDecimal ret1m;

    /**
     * 最近一个月基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret1mBm1;

    /**
     * 最近三个月收益率
     */
    private BigDecimal ret3m;

    /**
     * 最近三个月基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret3mBm1;

    /**
     * 最近半年收益率
     */
    private BigDecimal ret6m;

    /**
     * 最近半年基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret6mBm1;

    /**
     * 最近一年收益率
     */
    private BigDecimal ret1y;

    /**
     * 最近一年基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret1yBm1;

    /**
     * 最近两年收益率
     */
    private BigDecimal ret2y;

    /**
     * 最近两年基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret2yBm1;

    /**
     * 最近两年收益率（年化）
     */
    private BigDecimal ret2yA;

    /**
     * 最近两年基准指数收益率（年化），基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret2yBm1A;

    /**
     * 最近三年收益率
     */
    private BigDecimal ret3y;

    /**
     * 最近三年基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret3yBm1;

    /**
     * 最近三年收益率（年化）
     */
    private BigDecimal ret3yA;

    /**
     * 最近三年基准指数收益率（年化），基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret3yBm1A;

    /**
     * 最近四年收益率
     */
    private BigDecimal ret4y;

    /**
     * 最近四年基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret4yBm1;

    /**
     * 最近四年收益率（年化）
     */
    private BigDecimal ret4yA;

    /**
     * 最近四年基准指数收益率（年化），基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret4yBm1A;

    /**
     * 最近五年收益率
     */
    private BigDecimal ret5y;

    /**
     * 最近五年基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret5yBm1;

    /**
     * 最近五年收益率（年化）
     */
    private BigDecimal ret5yA;

    /**
     * 最近五年基准指数收益率（年化），基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal ret5yBm1A;

    /**
     * 今年以来收益率
     */
    private BigDecimal retYtd;

    /**
     * 今年以来基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal retYtdBm1;

    /**
     * 成立以来收益率
     */
    private BigDecimal retIncep;

    /**
     * 成立以来基准指数收益率，基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal retIncepBm1;

    /**
     * 成立以来收益率（年化）
     */
    private BigDecimal retIncepA;

    /**
     * 成立以来基准指数收益率（年化），基准指数bm1=沪深300（取决于基金设定的基准指数）
     */
    private BigDecimal retIncepBm1A;

    /**
     * 最近一年的夏普比率
     */
    private BigDecimal sharperatio1y;

    /**
     * 最近两年的夏普比率
     */
    private BigDecimal sharperatio2y;

    /**
     * 最近三年的夏普比率
     */
    private BigDecimal sharperatio3y;

    /**
     * 最近四年的夏普比率
     */
    private BigDecimal sharperatio4y;

    /**
     * 最近五年的夏普比率
     */
    private BigDecimal sharperatio5y;

    /**
     * 成立以来的夏普比率
     */
    private BigDecimal sharperatioIncep;

    /**
     * 最近一年的年化波动率
     */
    private BigDecimal stddev1y;

    /**
     * 最近两年的年化波动率
     */
    private BigDecimal stddev2y;

    /**
     * 最近三年的年化波动率
     */
    private BigDecimal stddev3y;

    /**
     * 最近四年的年化波动率
     */
    private BigDecimal stddev4y;

    /**
     * 最近五年的年化波动率
     */
    private BigDecimal stddev5y;

    /**
     * 最近十年的年化波动率
     */
    private BigDecimal stddev10y;

    /**
     * 成立以来的年化波动率
     */
    private BigDecimal stddevIncep;

    /**
     * 今年以来的年化波动率
     */
    private BigDecimal stddevYtd;

    /**
     * 最近一年的最大回撤
     */
    private BigDecimal maxdrawdown1y;

    /**
     * 最近两年的最大回撤
     */
    private BigDecimal maxdrawdown2y;

    /**
     * 最近四年的最大回撤
     */
    private BigDecimal maxdrawdown3y;

    /**
     * 最近四年的最大回撤
     */
    private BigDecimal maxdrawdown4y;

    /**
     * 最近五年的最大回撤
     */
    private BigDecimal maxdrawdown5y;

    /**
     * 最近十年的最大回撤
     */
    private BigDecimal maxdrawdown10y;

    /**
     * 成立以来的最大回撤
     */
    private BigDecimal maxdrawdownIncep;

    /**
     * 数据来源 1:tanpu，2:私募排排 3:指数
     */
    private Integer dataSources;

    private Date updateTime;

    private Date createTime;

    /**
     * 0:待上架  1：已上架 2：已下架
     */
    private Integer status;

    /**
     * 排序
     */
    private Integer sort;

    private Integer deleteTag;
}