package com.tanpu.fund.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.tanpu.common.auth.UserInfoThreadLocalHolder;
import com.tanpu.common.enums.BizEnums;
import com.tanpu.common.enums.FileTypeEnums;
import com.tanpu.common.model.file.req.FileQueryReq;
import com.tanpu.common.model.file.req.FileSaveReq;
import com.tanpu.common.model.file.resp.FileQueryResp;
import com.tanpu.common.model.fund.req.FundNoNavReq;
import com.tanpu.common.model.fund.resq.FundNoNavResp;
import com.tanpu.common.resp.CommonResp;
import com.tanpu.common.utils.SnowFlakeUtil;
import com.tanpu.fund.entity.generator.IfaImportedFundInfo;
import com.tanpu.fund.feign.user.FeignClientForFatools;
import com.tanpu.fund.mapper.generator.IfaImportedFundInfoMapper;
import com.tanpu.fund.service.IfaFundService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author: zyh
 * @date: 2021-03-13 2:46 下午
 * @description:
 */
@Service
public class IfaFundServieImpl implements IfaFundService {

    @Resource
    private SnowFlakeUtil snowFlakeUtil;
    @Resource
    private IfaImportedFundInfoMapper ifaImportedFundInfoMapper;
    @Resource
    private FeignClientForFatools feignClientForFatools;

    @Override
    public void fundnonavAdd(FundNoNavReq req) {
        IfaImportedFundInfo p = new IfaImportedFundInfo();
        BeanUtils.copyProperties(req, p);
        p.setId(snowFlakeUtil.uniqueLong());
        p.setRaiseStarttime(req.getRaiseStarttime());
        p.setRaiseEndtime(req.getRaiseEndtime());
        p.setIfaId(UserInfoThreadLocalHolder.getserInfo().getId());
        p.setCreateBy(UserInfoThreadLocalHolder.getserInfo().getId());
        p.setCreateTime(new Date());
        p.setDeleteTag(BizEnums.DeleteTag.tag_init);
        ifaImportedFundInfoMapper.insertSelective(p);

        //保存附件
        if (CollectionUtil.isNotEmpty(req.getProInfoDescPics())) {
            feignClientForFatools.fileSave(FileSaveReq.builder().refId(p.getId()).fileIds(req.getProInfoDescPics()).build());
        }

        if (CollectionUtil.isNotEmpty(req.getProFileDatas())) {
            feignClientForFatools.fileSave(FileSaveReq.builder().refId(p.getId()).fileIds(req.getProFileDatas()).build());
        }
    }

    @Override
    public void fundnonavEdit(FundNoNavReq req) {
        IfaImportedFundInfo p = new IfaImportedFundInfo();
        BeanUtils.copyProperties(req, p);
        p.setRaiseStarttime(req.getRaiseStarttime());
        p.setRaiseEndtime(req.getRaiseEndtime());
        p.setIfaId(UserInfoThreadLocalHolder.getserInfo().getId());
        p.setUpdateBy(UserInfoThreadLocalHolder.getserInfo().getId());
        p.setUpdateTime(new Date());
        p.setDeleteTag(BizEnums.DeleteTag.tag_init);
        ifaImportedFundInfoMapper.updateByPrimaryKey(p);

        //保存附件
        if (CollectionUtil.isNotEmpty(req.getProInfoDescPics())) {
            feignClientForFatools.fileSave(FileSaveReq.builder().refId(p.getId()).fileIds(req.getProInfoDescPics()).build());
        }

        if (CollectionUtil.isNotEmpty(req.getProFileDatas())) {
            feignClientForFatools.fileSave(FileSaveReq.builder().refId(p.getId()).fileIds(req.getProFileDatas()).build());
        }
    }

    @Override
    public FundNoNavResp fundnonavDetail(String id) {
        IfaImportedFundInfo ifaImportedFundInfo = ifaImportedFundInfoMapper.selectByPrimaryKey(id);
        FundNoNavResp p = new FundNoNavResp();
        BeanUtils.copyProperties(ifaImportedFundInfo, p);
        p.setRaiseStarttime(ifaImportedFundInfo.getRaiseStarttime() != null ? ifaImportedFundInfo.getRaiseStarttime().getTime() : null);
        p.setRaiseEndtime(ifaImportedFundInfo.getRaiseEndtime() != null ? ifaImportedFundInfo.getRaiseEndtime().getTime() : null);

        //详情图片
        {
            CommonResp<List<FileQueryResp>> commonResp = feignClientForFatools.fileQuery(FileQueryReq.builder().refId(id)
                    .fileType(FileTypeEnums.fundnonavdetail.name()).build());
            if (commonResp.isSuccess()) {
                p.setProInfoDescPics(commonResp.getAttributes());
            }
        }

        //资料
        {
            CommonResp<List<FileQueryResp>> commonResp = feignClientForFatools.fileQuery(FileQueryReq.builder().refId(id)
                    .fileType(FileTypeEnums.fundnonavdatas.name()).build());
            if (commonResp.isSuccess()) {
                p.setProFileDatas(commonResp.getAttributes());
            }
        }

        return p;
    }

}
