package com.tanpu.fund.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.page.PageMethod;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.tanpu.common.enums.Constant;
import com.tanpu.common.enums.product.ProductEnums;
import com.tanpu.common.model.Page;
import com.tanpu.common.model.Pageable;
import com.tanpu.common.model.product.FundInfoVO;
import com.tanpu.common.model.product.req.ExternalReq;
import com.tanpu.common.model.product.req.NetReq;
import com.tanpu.common.model.product.req.ProductInfoReq;
import com.tanpu.common.model.product.req.ProductListReq;
import com.tanpu.common.model.product.resp.*;
import com.tanpu.common.model.tanpuroom.Type;
import com.tanpu.common.model.tanpuroom.resp.SysConstantVO;
import com.tanpu.common.model.user.resp.SysConstantResp;
import com.tanpu.common.resp.CommonResp;
import com.tanpu.common.utils.BigDecimalUtil;
import com.tanpu.fund.entity.generator.*;
import com.tanpu.fund.enums.ProTypeEnums;
import com.tanpu.fund.feign.diagnose.FeignClientForDiagnose;
import com.tanpu.fund.feign.user.FeignClientForFatools;
import com.tanpu.fund.mapper.generator.*;
import com.tanpu.fund.mapper.generator.custom.FundInfoCustomMapper;
import com.tanpu.fund.service.ProductPrivateService;
import com.tanpu.fund.utils.LongUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.tanpu.common.utils.BigDecimalUtil.multiply100;

/**
 * @author: zhoupeng
 * <p>
 * =========================================
 * =========================================
 * ========                         ========
 * =======   ==========   =======    =======
 * ======        =====    ==   ====   ======
 * =====        =====     ==    ===    =====
 * =====       =====      ==   ===     =====
 * =====      =====       == ===       =====
 * ======    ==========   ==          ======
 * =======                           =======
 * =========================================
 * =========================================
 * <p>
 * @email: zhoupeng_08@163.com
 */
@Service
@Slf4j
public class ProductPrivateServiceImpl implements ProductPrivateService, Constant {

    @Value("${company.oss.prefix}")
    private String filePrefix;

    @Resource
    private FundInfoMapper fundInfoMapper;

    @Resource
    private CompanyInfoMapper companyInfoMapper;

    @Resource
    private PersonnelInfoMapper personnelInfoMapper;

    @Resource
    private CompanyTnaPersonnelMapper companyTnaPersonnelMapper;

    @Resource
    private FundManagerMappingMapper fundManagerMappingMapper;

    @Resource
    private FundCountMapper fundCountMapper;

    @Resource
    private FundDistributionMapper fundDistributionMapper;

    @Resource
    private PersonCompanyPositionMappingMapper personCompanyPositionMappingMapper;

    @Resource
    private FeignClientForDiagnose diagnose;

    @Resource
    private FundInfoCustomMapper fundInfoCustomMapper;

    @Resource
    private FeignClientForFatools feignClientForFatools;

    @Override
    public Page<ProductInfoVO> getProductList(ProductInfoReq req) {
        return null;
    }

    @Override
    public ProductInfoVO getProductDetail(String id) {
        return null;
    }

    @Override
    public List<ProductInfoVO> getProductList(List<String> ids) {
        return null;
    }

    @Override
    public Page<ProductInfoVO> getNotTanpuProduct(ProductInfoReq req) {
        return null;
    }

    @Override
    public UnitNetVO getUnitNet(String id, String ratioId, String type) {
        return null;
    }

    @Override
    public UnitNetVO getProfitTrend(String id, String ratioId, String type, Long beginTime, Long endTime) {
        return null;
    }

    @Override
    public Page<NetVO> getNetList(NetReq req) {
        return null;
    }

    @Override
    public List<BonusRatioVO> getBonusRatio(String id) {
        FundDistributionExample example = new FundDistributionExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("distribute_date desc");
        List<FundDistribution> list = this.fundDistributionMapper.selectByExample(example);
        return list.stream().filter(
                l -> l.getDistributeDate() != null && (l.getDistributeType() == 1 || l.getDistributeType() == 2))
                .map(l -> {
                    BonusRatioVO vo = new BonusRatioVO();
                    vo.setType(l.getDistributeType());
                    vo.setTime(l.getDistributeDate().getTime());
                    vo.setDistribution(BigDecimalUtil.toString(l.getDistribution(), 2));
                    return vo;
                }).collect(Collectors.toList());
    }

    @Override
    public List<TrackRecordVO> getTrackRecord(String id) {
        return null;
    }

    @Override
    public Page<TrackNetVO> getTrackNetList(String id, Pageable page) {
        return null;
    }

    @Override
    public List<RiskRatingVO> getRiskRating(String id) {
        return null;
    }

    @Override
    public List<FundManagerVO> getFundManager(String id, String ifaId) {
        FundManagerMappingExample example = new FundManagerMappingExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("management_start_date desc,id desc");

        List<String> list = this.fundManagerMappingMapper.selectByExample(example).stream()
                .map(FundManagerMapping::getFundManagerId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        example.clear();
        example.createCriteria().andFundManagerIdIn(list);
        List<FundManagerMapping> managerMappings = this.fundManagerMappingMapper.selectByExample(example);

        List<String> fundIds = managerMappings.stream().map(FundManagerMapping::getFundId).distinct()
                .collect(Collectors.toList());

        fundIds.add(id);

        FundCountExample example2 = new FundCountExample();
        example2.createCriteria()
                .andFundIdIn(fundIds)
                .andRetIncepIsNotNull()
                .andDeleteTagEqualTo(ZERO_NUM);
        example2.setOrderByClause("ret_incep desc");
        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example2);
        Map<String, FundCount> fundCountMap = fundCounts.stream()
                .collect(Collectors.toMap(FundCount::getFundId, f -> f));

        Map<String, List<FundManagerMapping>> map = managerMappings.stream()
                .collect(Collectors.groupingBy(FundManagerMapping::getFundManagerId));

        // 获取基金基本信息
        FundInfoExample example3 = new FundInfoExample();
        example3.createCriteria().andIdIn(fundIds).andDeleteTagEqualTo(ZERO_NUM);
        Map<String, FundInfo> infoMap = this.fundInfoMapper.selectByExample(example3).stream()
                .collect(Collectors.toMap(FundInfo::getId, f -> f));

        // 获取详情信息
        PersonnelInfoExample example1 = new PersonnelInfoExample();
        example1.createCriteria().andIdIn(list).andDeleteTagEqualTo(ZERO_NUM);
        List<PersonnelInfo> infos = this.personnelInfoMapper.selectByExampleWithBLOBs(example1);

        Set<String> isCheckSet = getIsCheck(ifaId, fundIds);

        // 基金经理
        return infos.stream().map(i -> {
            FundManagerVO vo = new FundManagerVO();
            vo.setManagerId(i.getId());
            vo.setManagerName(i.getPersonnelName());
            vo.setJobTime(i.getInvestmentExperience());
            vo.setPersonalProfile(i.getProfile());
            vo.setEducation(i.getEducation());


            List<FundManagerMapping> managerMappingList = map.get(i.getId());
            if (CollectionUtils.isNotEmpty(managerMappingList)) {
                String fundId = managerMappingList.get(0).getFundId();
                vo.setId(fundId);
                if (managerMappingList.get(0).getManagementStartDate() != null) {
                    vo.setManagementStartDate(managerMappingList.get(0).getManagementStartDate().getTime());
                }
                vo.setIsCheck(0);
                if (isCheckSet.contains(vo.getId())) {
                    vo.setIsCheck(1);
                }

                FundInfo info = infoMap.get(fundId);
                vo.setFundName(info.getFundShortName());
                vo.setRatioId(ProductEnums.IndexEnum.getValue(info.getPrimaryBenchmarkId()));
                FundCount fundCount = fundCountMap.get(fundId);
                if (fundCount != null) {
                    vo.setRetIncep(multiply100(fundCount.getRetIncep()));
                    vo.setNet(new Net(fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime(),
                            BigDecimalUtil.toString(fundCount.getCumulativeNav(), 4), BigDecimalUtil.toString(fundCount.getCumulativeNav(), 4)));
                }

                if (infoMap.get(fundId) != null) {
                    FundInfo fundInfo = infoMap.get(fundId);
                    vo.setTime(LongUtil.timeLong(fundInfo.getRegisterDate()));
                    vo.setInceptionDate(LongUtil.timeLong(fundInfo.getInceptionDate()));
                }
                // 设置代表基金id 其他基金信息
                getDelegateById(i.getId(), map, infoMap, fundCountMap, vo, isCheckSet);
            }
            return vo;
        }).collect(Collectors.toList());
    }

    private void getDelegateById(String personalId, Map<String, List<FundManagerMapping>> map,
                                 Map<String, FundInfo> infoMap, Map<String, FundCount> fundCountMap,
                                 FundManagerVO vo, Set<String> isCheckSet) {
        if (CollectionUtils.isNotEmpty(map.get(personalId))) {
            List<FundManagerMapping> mappings = map.get(personalId);

            if (CollectionUtils.isEmpty(mappings)) {
                return;
            }

            List<FundCount> list = mappings.stream()
                    .filter(mapping -> fundCountMap.containsKey(mapping.getFundId()))
                    .map(m -> fundCountMap.get(m.getFundId()))
                    .distinct()
                    .sorted(Comparator.comparing(FundCount::getRetIncep).reversed())
                    .collect(Collectors.toList());

            if (CollectionUtils.isNotEmpty(list)) {
                FundCount fundCount = list.get(0);
                vo.setRetIncep(multiply100(fundCount.getRetIncep()));
                vo.setInceptionDate(fundCount.getPriceDate() == null ? null : fundCount.getPriceDate().getTime());
            }

            vo.setFundInfoVOS(list.stream().map(l -> {
                FundInfoVO infoVO = new FundInfoVO();
                String fundId = l.getFundId();
                infoVO.setIsCheck(isCheckSet.contains(fundId) ? 1 : 0);
                infoVO.setFundId(fundId);
                if (infoMap.get(fundId) != null) {
                    FundInfo fundInfo = infoMap.get(fundId);
                    infoVO.setFundName(fundInfo.getFundShortName());
                    infoVO.setRatioId(ProductEnums.IndexEnum.getValue(fundInfo.getPrimaryBenchmarkId()));
                    if (fundInfo.getInceptionDate() != null) {
                        infoVO.setInceptionDate(fundInfo.getInceptionDate().getTime());
                    }
                    infoVO.setBonusTypeName(ProTypeEnums.getName(fundInfo.getFundType()));
                    infoVO.setBonusType(fundInfo.getFundType());
                }

                if (l.getEndDate() != null) {
                    infoVO.setNet(Net.builder().netDate(l.getEndDate().getTime()).netValue(BigDecimalUtil.toString(l.getNetNav(), 4)).cumulativeNav(BigDecimalUtil.toString(l.getCumulativeNav(), 2)).build());
                }
                infoVO.setRet1y(multiply100(l.getRet1y()));
                infoVO.setRetIncep(multiply100(l.getRetIncep()));
                infoVO.setYearProfit(multiply100(l.getRetIncepA()));

                return infoVO;
            }).collect(Collectors.toList()));
        }
    }

    @Override
    public FundRecordVO getFundRecord(String id) {
        return null;
    }

    @Override
    public FundCompanyVO getFundCompany(String id, String ifaId, Pageable page) {
        CompanyInfo companyInfo = this.companyInfoMapper.selectByPrimaryKey(id);
        if (companyInfo == null) {
            return new FundCompanyVO();
        }

        CompanyTnaPersonnelExample example = new CompanyTnaPersonnelExample();
        example.createCriteria().andCompanyIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        List<CompanyTnaPersonnel> companyTnaPersonnels = this.companyTnaPersonnelMapper.selectByExample(example);

        FundCompanyVO vo = new FundCompanyVO();
        vo.setCompanyName(companyInfo.getCompanyName());
        vo.setCompanyId(companyInfo.getId());
        vo.setCompanyProfile(companyInfo.getCompanyProfile());
        vo.setImg(filePrefix + companyInfo.getLogo());

        if (CollectionUtils.isNotEmpty(companyTnaPersonnels)) {
            CompanyTnaPersonnel companyTnaPersonnel = companyTnaPersonnels.get(0);
            vo.setTotalMoney(companyTnaPersonnel.getTotalAssetSize() == null ? null
                    : companyTnaPersonnel.getTotalAssetSize().toString());
            vo.setTotalFund(companyTnaPersonnel.getFundNum());
        }

        // 基金经理信息
        PersonCompanyPositionMappingExample example2 = new PersonCompanyPositionMappingExample();
        example2.createCriteria().andCompanyIdEqualTo(companyInfo.getId()).andPersonnelTypeEqualTo(1);
        List<PersonCompanyPositionMapping> mappings = this.personCompanyPositionMappingMapper.selectByExample(example2);
        if (CollectionUtils.isNotEmpty(mappings)) {
            List<PersonnelInfo> personnelInfos = getPersonnelInfos(mappings.stream().map(PersonCompanyPositionMapping::getPersonnelId).collect(Collectors.toList()));
            vo.setTypes(personnelInfos.stream().map(personnelInfo ->
                    Type.builder().id(personnelInfo.getId()).value(personnelInfo.getPersonnelName()).build()
            ).collect(Collectors.toList()));
        }

        // 其他基金信息
        otherFundInfo(vo, id, ifaId, page);

        vo.setIsCheck(0);
        if (StringUtils.isNotEmpty(ifaId)) {
            List<String> attributes = diagnose.getFundCollectInfo(ExternalReq.builder().ids(Lists.newArrayList(vo.getFundId())).ifaId(ifaId).build()).getAttributes();
            if (!CollectionUtils.isEmpty(attributes)) {
                Set<String> checkSet = Sets.newHashSet(attributes);
                vo.setIsCheck(checkSet.contains(vo.getFundId()) ? 1 : 0);
            }
        }

        return vo;
    }

    private List<PersonnelInfo> getPersonnelInfos(List<String> ids) {
        PersonnelInfoExample example1 = new PersonnelInfoExample();
        example1.createCriteria().andIdIn(ids);
        return this.personnelInfoMapper.selectByExample(example1);
    }

    private Map<String, PersonnelInfo> getPersonnelInfoMap(List<String> ids) {
        List<PersonnelInfo> personnelInfos = getPersonnelInfos(ids);
        return personnelInfos.stream().collect(Collectors.toMap(PersonnelInfo::getId, s -> s));
    }

    private void otherFundInfo(FundCompanyVO vo, String id, String ifaId, Pageable page) {

        FundInfoExample example1 = new FundInfoExample();
        example1.createCriteria().andTrustIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        List<FundInfo> fundInfos = this.fundInfoMapper.selectByExample(example1);
        List<String> list = fundInfos.stream().map(FundInfo::getId).collect(Collectors.toList());
        Map<String, FundInfo> mapFundInfo = fundInfos.stream().collect(Collectors.toMap(FundInfo::getId, f -> f));

        if (CollectionUtils.isNotEmpty(list)) {
            FundCountExample example2 = new FundCountExample();
            example2.createCriteria().andFundIdIn(list)
                    .andDeleteTagEqualTo(ZERO_NUM)
                    .andRet1yBm1IsNotNull();
            example2.setOrderByClause("ret_incep desc,ret_incep_a desc");

            com.github.pagehelper.Page<Object> objectPage = PageMethod.startPage(page.getPageNumber(), page.getPageSize());

            List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example2);

            if (CollectionUtils.isNotEmpty(fundCounts)) {

                Set<String> isCheckSet = getIsCheck(ifaId, list);

                fundCounts.sort(Comparator.comparing(FundCount::getRetIncep).reversed());
                FundCount fundCount = fundCounts.get(0);

                FundManagerMappingExample example = new FundManagerMappingExample();
                example.createCriteria().andFundIdIn(fundCounts.stream().map(FundCount::getFundId).collect(Collectors.toList()));
                Map<String, String> fundIdPersonals = this.fundManagerMappingMapper.selectByExample(example)
                        .stream()
                        .collect(Collectors.groupingBy(FundManagerMapping::getFundId, Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0).getFundManagerId())));

                Map<String, PersonnelInfo> personnelInfoMap = getPersonnelInfoMap(Lists.newArrayList(fundIdPersonals.values()));

                FundInfo info = mapFundInfo.get(fundCount.getFundId());
                vo.setTime(info.getRegisterDate() == null ? null : info.getRegisterDate().getTime());
                vo.setFundId(fundCount.getFundId());
                vo.setRatioId(ProductEnums.IndexEnum.getValue(info.getPrimaryBenchmarkId()));
                vo.setInceptionDate(info.getInceptionDate() == null ? null : info.getInceptionDate().getTime());
                vo.setFundName(info.getFundShortName());
                vo.setNet(new Net(fundCount.getEndDate().getTime(), BigDecimalUtil.toString(fundCount.getCumulativeNav(), 4), BigDecimalUtil.toString(fundCount.getCumulativeNav(), 4)));
                vo.setRetIncep(multiply100(fundCount.getRetIncep()));
                vo.setFundInfoVOS(fundCounts.stream().map(f -> {
                    FundInfoVO vo1 = new FundInfoVO();
                    String fundId = f.getFundId();
                    vo1.setFundId(fundId);
                    vo1.setIsCheck(isCheckSet.contains(fundId) ? 1 : 0);
                    vo1.setBonusTypeName(ProTypeEnums.getName(mapFundInfo.get(fundId).getFundType()));
                    vo1.setBonusType(mapFundInfo.get(fundId).getFundType());
                    vo1.setRetIncep(multiply100(f.getRetIncep()));
                    vo1.setYearProfit(multiply100(f.getRetIncepA()));
                    vo1.setNet(new Net(f.getEndDate().getTime(), BigDecimalUtil.toString(f.getNetNav(), 4), BigDecimalUtil.toString(f.getCumulativeNav(), 4)));
                    vo1.setFundName(mapFundInfo.get(fundId).getFundShortName());
                    if (fundIdPersonals.containsKey(fundId)) {
                        if (personnelInfoMap.containsKey(fundIdPersonals.get(fundId))) {
                            vo1.setManagerType(Type.builder().id(fundId).value(personnelInfoMap.get(fundIdPersonals.get(fundId)).getPersonnelName()).build());
                        }
                    }
                    if (mapFundInfo.get(fundId).getInceptionDate() != null) {
                        vo1.setInceptionDate(mapFundInfo.get(fundId).getInceptionDate().getTime());
                    }

                    vo1.setRet1y(multiply100(f.getRet1y()));

                    vo1.setRatioId(ProductEnums.IndexEnum.getValue(info.getPrimaryBenchmarkId()));

                    return vo1;
                }).collect(Collectors.toList()));
                vo.setPage(new Page<>(objectPage, vo.getFundInfoVOS()));
            }
        }
    }

    private Set<String> getIsCheck(String ifaId, List<String> list) {
        log.info("入参信息 ifa : {},基金id : {}", ifaId, JSON.toJSONString(list));
        Set<String> isCheckSet;

        if (StringUtils.isEmpty(ifaId)) {
            isCheckSet = new HashSet<>(0);
        } else {
            List<String> attributes = diagnose.getFundCollectInfo(ExternalReq.builder().ids(list).ifaId(ifaId).build()).getAttributes();

            log.info("getFundCollectInfo 返回信息:{}", JSON.toJSONString(attributes));

            if (CollectionUtils.isEmpty(attributes)) {
                isCheckSet = new HashSet<>(0);
            } else {
                isCheckSet = Sets.newHashSet(attributes);
            }
        }
        log.info("返回参数 {}", JSON.toJSONString(isCheckSet));
        return isCheckSet;
    }

    @Override
    public FundCompanyVO getPrivateFundCompany(String id) {
        return null;
    }

    @Override
    public List<ProductInfoVO> getProductInfolist(ProductListReq req) {
        return null;
    }

    @Override
    public List<ProductInfoVO> getNotTanpuProductInfolist(ProductListReq req) {
        return null;
    }

    @Override
    public DynamicRetreatVO getDynamicRetreat(String id) {
        return null;
    }

    @Override
    public ProductInfoVO getPrivateDetail(String id) {
        return null;
    }

    @Override
    public List<TrackRecordVO> getHistoryprofit(String id) {
        return null;
    }

    @Override
    public Page<TrackNetVO> getPrivateFundTrackNetList(String id, Pageable page) {
        return null;
    }

    @Override
    public List<RiskRatingVO> getPrivateFundRiskRating(String id) {
        return null;
    }

    @Override
    public List<ProductInfoVO> getPrivateFundList(List<String> ids) {
        return null;
    }

    @Override
    public List<FundInfoSimpleListResp> getSimpleList(List<String> idList) {
        return null;
    }

    @Override
    public List<FundInfoSimpleListResp> getPrivateSimpleList(List<String> idList) {
        return null;
    }

    @Override
    public List<FundSameResp> querySamefund(String fundId) {
        return null;
    }

    @Override
    public List<FundSameResp> getRemmendPrivateFundList() {
        return null;
    }

    @Override
    public List<Type> getSimplePrivateFundList(List<String> list) {
        return fundInfoCustomMapper.getSimplePrivateFundList(list.stream().collect(Collectors.joining("','", "'", "'")));
    }

    @Override
    public List<SysConstantVO> getSubstrategyList() {
        // tampsubstrategy 私募基金子策略
        CommonResp<List<SysConstantResp>> resp = feignClientForFatools.queryLabels("tampsubstrategy");
        if (!resp.isSuccess() || CollectionUtils.isEmpty(resp.getAttributes())) {
            return Lists.newArrayList();
        }
        // 排除不需要的策略（白名单产品策略）
        List<String> excludeCodes = Lists.newArrayList("9010", "9020", "9030", "9040", "9050", "7030");
        return resp.attributes.stream().filter(v -> !excludeCodes.contains(v)).map(item -> {
            SysConstantVO vo = new SysConstantVO();
            vo.setConstantCode(item.getConstantCode());
            vo.setConstantName(item.getConstantName());
            vo.setConstantValue(item.getConstantValue());
            return vo;
        }).collect(Collectors.toList());
    }
}
